/*--

THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Copyright (C) 1999  Microsoft Corporation.  All rights reserved.

Module Name:

    ksetpw.c

Abstract:

    Set a user's password using the
    Kerberos Change Password Protocol (I-D) variant for Windows 2000

--*/

#include <stdio.h>
#include <sys/types.h>
#ifdef HAVE_PWD_H
#include <pwd.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <krb5.h>

static const char rcsid[] = "$Id$";

#define P1 "Enter new password: "
#define P2 "Enter it again: "

int main(int argc, char *argv[])
{
    krb5_error_code ret;
    krb5_context context;
    char *pname = NULL;
    char *targpname = NULL;
    krb5_ccache ccache;
    krb5_principal targprinc;
    char pw[1024];
    int pwlen;
    int result_code;
    krb5_data result_code_string, result_string;
   
    if (argc != 2) {
	fprintf(stderr, "usage: %s <principal>\n", argv[0]);
	exit(1);
    }

    targpname = argv[1];

    if (ret = krb5_init_context(&context)) {
	com_err(argv[0], ret, "initializing kerberos library");
	exit(1);
    }

    if (ret = krb5_cc_default(context, &ccache)) {
	if (ret == KRB5_CC_NOTFOUND)
	    printf("No Kerberos credentials; kinit then try again.\n");
	else
	    com_err(argv[0], ret, "opening default ccache");
	exit(1);
    }

    if (ret = krb5_parse_name(context, targpname, &targprinc)) {
	com_err(argv[0], ret, "while parsing target principal");
	exit(1);
    }
		
    if (ret = krb5_unparse_name(context, targprinc, &targpname)) {
	com_err(argv[0], ret, "while parsing target principal");
	exit(1);
    }

    printf("Changing password for %s\n", targpname);
   
    pwlen = sizeof(pw);
    if (ret = krb5_read_password(context, P1, P2, pw, &pwlen)) {
	com_err(argv[0], ret, "while reading password");
	exit(1);
    }

    if (ret = krb5_set_password(context, ccache, pw, targprinc,
				&result_code, &result_code_string,
				&result_string)) {
	com_err(argv[0], ret, "changing password");
	exit(1);
    }

    if (result_code) {
	printf("%.*s%s%.*s\n",
	       result_code_string.length, result_code_string.data,
	       result_string.length?": ":"",
	       result_string.length, result_string.data);
	exit(2);
    }

    free(result_string.data);
    free(result_code_string.data);

    printf("Password changed.\n");
    exit(0);
}
