/*--

THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Copyright (C) 1999  Microsoft Corporation.  All rights reserved.

Module Name:

    locate.c

Abstract:

    Locate win2000 DCs based on DNS SRV RR's registered.

--*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif
#include "locator.h"

static const char rcsid[] = "$Id$";

char *argv0;

void usage(void)
{
    printf("Usage: %s [-S %s] [-P %s] [-fn] domain\n",
	   argv0, LDAP_SRV, TCP_PROTO);
    exit(0);
}

char *nameoraddr(int praddr, char *host)
{
    struct hostent *hp;
    
    if (praddr) {
	if (hp = gethostbyname(host)) {
	    if (hp->h_addrtype == AF_INET) {
		return inet_ntoa(*(struct in_addr *)hp->h_addr);
	    }
	    else
		return host;
	}
	else
	    return host;
    }
    else
	return host;
}

int main(int argc, char *argv[])
{
    int i, ch;
    int nsrv;
    int full = 0, addr = 0;
    PSRV_LOCATION *psrv;
    char *domain, *srv = LDAP_SRV, *proto = TCP_PROTO;
    
    if (argv0 = strrchr(argv[0], '/'))
	argv0++;
    else
	argv0 = argv[0];

    while ((ch = getopt(argc, argv, "S:P:fn?")) != -1) {
	switch (ch) {
	case 'S':
	    srv = optarg;
	    break;
	case 'P':
	    proto = optarg;
	    break;
	case 'f':
	    full = 1;
	    break;
	case 'n':
	    addr = 1;
	    break;
	case '?':
	default:
	    usage();
	    break;
	}
    }
    
    argc -= optind;
    argv += optind;

    if (argc < 1) {
	usage();
    }
    
    domain = argv[0];
    
    if (locate_srv(domain, srv, proto, &psrv, &nsrv) != 0) {
	fprintf(stderr, "Failed to locate service\n");
	exit(1);
    }
    
    if (full)
	printf("%d service locations\n", nsrv);
    for (i = 0; i < nsrv; i++) {
	if (full)
	    printf("%d: %s [%d %d]\n", i,
		   nameoraddr(addr, psrv[i]->srv_host), psrv[i]->srv_port,
		   psrv[i]->srv_weight, psrv[i]->srv_priority);
	else
	    printf("%s ", nameoraddr(addr, psrv[i]->srv_host));
    }
    if (!full)
	printf("\n");

    exit(0);
}


