/*--

THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Copyright (C) 1999  Microsoft Corporation.  All rights reserved.

Module Name:

    locator.c

Abstract:

    Locate SRV records in DNS

--*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include <string.h>
#include "locator.h"

static const char rcsid[] = "$Id$";

int locate_srv(
    const char *domain,
    const char *srv,
    const char *protocol,
    PSRV_LOCATION **ploc,
    int *pnum
    )
{
    char service[128], host[128];
    int len;
    unsigned char reply[1024];
    int status;
    unsigned char *p;
    int priority, weight;
    u_short port;
    PSRV_LOCATION *locs;
    int num_locs = 0;
    
    if (!protocol)
	protocol = TCP_PROTO;
    
#ifdef HAVE_SNPRINTF
    snprintf(service, sizeof(service), "%s.%s.%s.",
	     srv, protocol, domain);
#else
    sprintf(service, "%s.%s.%s.",
	     srv, protocol, domain);
#endif

    len = res_search(service, C_IN, T_SRV, reply, sizeof(reply));
    if (len >=0) {

	/* Parse out query */
	p = reply;
	p += sizeof(HEADER);
	status = dn_expand(reply, reply + len, p, host, sizeof(host));
	if (status < 0) {
	    return -1;
	}
	p += status;
	p += 4;

	while (p < reply + len) {
	    int type, class, ttl, size;
	    status = dn_expand(reply, reply + len, p, host, sizeof(host));
	    if (status < 0) {
		if (num_locs)
		    break;
		else
		    return -1;
	    }
	    p += status;
	    type = (p[0] << 8) | p[1];
	    p += 2;
	    class = (p[0] << 8) | p[1];
	    p += 2;
	    ttl = (p[0] << 24) | (p[1] << 16) | (p[2] << 8) | p[3];
	    p += 4;
	    size = (p[0] << 8) | p[1];
	    p += 2;
	    if (type == T_SRV) {
		status = dn_expand(reply, reply + len, p + 6, host, sizeof(host));
		if (status < 0) {
		    return -1;
		}
		priority = (p[0] << 8) | p[1];
		weight = (p[2] << 8) | p[3];
		port = (p[4] << 8) | p[5];

		if (num_locs)
		    locs = realloc(locs, sizeof(PSRV_LOCATION)*(num_locs+1));
		else
		    locs = malloc(sizeof(PSRV_LOCATION));
		locs[num_locs] = malloc(sizeof(SRV_LOCATION));
		locs[num_locs]->srv_host = strdup(host);
		locs[num_locs]->srv_port = port;
		locs[num_locs]->srv_weight = weight;
		locs[num_locs]->srv_priority = priority;
		num_locs++;
		
#ifdef DBG
		printf("%s:%d [%d:%d]\n", host, port, weight, priority);
#endif
		/* XXX This should really be sorted by priority */
		p += size;
	    }
	}
    }
    *ploc = locs;
    *pnum = num_locs;
    return 0;
}
